# A Julia template for Bitemporal Data Management based on SearchLight.jl
This package provides generic functions for CRUD of bitemporal aggregate entities.
Serializing and deserializing is provided by packages JSON and ToStruct.

The package's application based transaction logic comprises reified transactions such that
even pending transactions' data are persistent. Pending transactions can therefore be suspended, resumed
and delegated: passed between users within workflows.

## Twodimensional intervals representing panes of validity

Bitemporal data management models temporal references of data, such as

- a contract's begin of validity,i.e. from when on it shall be executed
- a contract's date of signature, or in a computational context: the timestamp of its final (committed) insertion into a database.

For instance, we agreed

* on Monday on s.th. we call the blue agreement beginning next year,
* on Tuesday on the yellow one beginning two years later, and
* on Wednesday on the red one instead, beginning already one year later
* and leave it at that.

Our model is to provide answers to questions like

"What did we think at *transaction time* what we agreed upon as of *referenced time*"

Reference time and transaction time can be seen as
constituting a point which is contained in a
rectangle of validity and thus identifies a version.

The first idea of this approach is spatial representation of the two temporal dimensions.
The above course of agreements would be described by sets of adjacent rectangles,
reference time running horizontally, transaction time vertically and versions eperesented by colour.

![Areas of Validity](../assets/ValidityRectangles.png)

Let's denote these rectangles by 2dimensional intervals, where *lower* stands for the start time, which is included, and *upper* for the end time which is not included. *upper* = infinity stands for an unspecified end time. We shall use use *v* to denote the version, i.e. the state of the object's attribute values that persists in the interval of validity.

While under this interpretation

* the first dimension allows references to past or future points in time just as human language does,
* the second dimension allows
  * only references to the past for *lower* as for the assertion to even exist in the data base it must have been entered at some past point of time; and
  * references to the past and to infinity for *upper* as the time of its revocation can be past as well as yet unspecified.

```@raw html
<pre><code>[d<sub>lower</sub> ,d<sub>upper</sub>) [w<sub>lower</sub> ,w<sub>upper</sub>)</pre></code>
```

The first, blue, part in the above example is open ended in both dimensions, no end of world validity is intended and no mutation of the data base has been done yet,

```@raw html
<pre><code>(v<sub>blue</sub> , [d<sub>lower</sub> ,∞ ) [w<sub>lower</sub> ,∞) )</pre></code>
```

The second part, the transition to yellow can be denoted like so:
```@raw html
<pre><code>let
    s<sub>0</sub> = { (v<sub>blue</sub> [d<sub>0</sub> ,∞ ) [w<sub>0</sub> ,∞ ) ) } 
    t<sub>1</sub> = (d<sub>1</sub>,w<sub>1</sub>) where d<sub>1</sub> > d<sub>0</sub>, w<sub>1</sub> > w<sub>0</sub>
then mutation of s<sub>0</sub> to yellow per t<sub>1</sub> equals
    { ( v<sub>blue</sub>, [d<sub>0</sub>,d<sub>1</sub>) [w<sub>0</sub>,∞) [d<sub>1</sub>,∞) [w<sub>0</sub>,w<sub>1</sub>) ),
      ( v<sub>yellow</sub>, [d<sub>1</sub>,∞) [w<sub>1</sub>,∞) ) }
</pre></code>
```

The third part, the mutation overwriting yellow with red, can be denoted like so:

```@raw html
<pre><code>let 
    s<sub>1</sub> =
    { ( v<sub>blue</sub>, [d<sub>0</sub>,d<sub>1</sub>) [w<sub>0</sub>,∞) [d<sub>1</sub>,∞) [w<sub>0</sub>,w<sub>1</sub>) ),
      ( v<sub>yellow</sub>, [d<sub>1</sub>,∞) [w<sub>1</sub>,∞) ) }
    t<sub>2</sub> = (d<sub>2</sub>,w<sub>2</sub>) where d<sub>2</sub> > d<sub>1</sub>, w<sub>2</sub> < w<sub>1</sub>
then mutation of s<sub>1</sub> to red per t<sub>2</sub> equals
    { ( v<sub>blue</sub>, [d<sub>0</sub>,d<sub>1</sub>) [w<sub>0</sub>,∞) [d<sub>1</sub>,d<sub>2</sub>) [w<sub>0</sub>,w<sub>1</sub>) [d<sub>2</sub>,∞) [w<sub>0</sub>,w<sub>2</sub>) ),
      ( v<sub>yellow</sub>, [d<sub>1</sub>,d<sub>2</sub>,) [w<sub>1</sub>,∞) ),
      ( v<sub>red</sub>, [d<sub>2</sub>,∞) [w<sub>2</sub>,∞) ) }
</pre></code>
```

## Representing bitemporal aggregations

When we look at bitemporal aggregates we find components that can change independently at different times, like for instance an insurance contract with subobjects like a reference to the premium payer, tariff parameters like height of premium, modes of payment, date of last payment etc.etc..
Thus we find that these subobjects need their own intervals of validity; but because the aggregate changes whenever a component changes, the aggregate version can share its intervals with the changing components. Let's enhance our example such that we have three components of which

* one is added at creation as of the blue version of the aggregate, but deleted as of the yellow one
* another added at creation as of the yellow version of the aggregate, but deleted as of the red one and
* a third, is added at creation as of the blue version of the aggregate, and kept as of the yellow and red versions

```@raw html
<pre><code>Let 
h = { ( v<sub>blue</sub>, [d<sub>0</sub>,d<sub>1</sub>) [w<sub>0</sub>,∞) [d<sub>1</sub>,d<sub>2</sub>) [w<sub>0</sub>,w<sub>1</sub>) [d<sub>2</sub>,∞) [w<sub>0</sub>,w<sub>2</sub>) ),
      ( v<sub>yellow</sub>, [d<sub>1</sub>,d<sub>2</sub>,) [w<sub>1</sub>,∞) ),
      ( v<sub>red</sub>, [d<sub>2</sub>,∞) [w<sub>2</sub>,∞) ) }
c<sub>i</sub> = ( v<sub>blue</sub>, [d<sub>0</sub>,d<sub>1</sub>) [w<sub>0</sub>,∞) [d<sub>1</sub>,d<sub>2</sub>) [w<sub>0</sub>,w<sub>1</sub>) [d<sub>2</sub>,∞) [w<sub>0</sub>,w<sub>2</sub>) )
c<sub>2</sub> = ( v<sub>yellow</sub>, [d<sub>1</sub>,d<sub>2</sub>,) [w<sub>1</sub>,∞) )
c<sub>3</sub> = ( v<sub>blue</sub>, [d<sub>0</sub>,d<sub>1</sub>) [w<sub>0</sub>,∞) [d<sub>1</sub>,d<sub>2</sub>) [w<sub>0</sub>,w<sub>1</sub>) [d<sub>2</sub>,∞) [w<sub>0</sub>,w<sub>2</sub>) )( v<sub>yellow</sub>, [d<sub>1</sub>,d<sub>2</sub>,) [w<sub>1</sub>,∞)(v<sub>red</sub>, [d<sub>2</sub>,∞) [w<sub>2</sub>,∞) )
</pre></code>
```

This can be simplified by substituting the interval sets by references to the versions they are shared with.

```@raw html
<pre><code>Let 
h = { ( v<sub>blue</sub>, [d<sub>0</sub>,d<sub>1</sub>) [w<sub>0</sub>,∞) [d<sub>1</sub>,d<sub>2</sub>) [w<sub>0</sub>,w<sub>1</sub>) [d<sub>2</sub>,∞) [w<sub>0</sub>,w<sub>2</sub>) ),
      ( v<sub>yellow</sub>, [d<sub>1</sub>,d<sub>2</sub>,) [w<sub>1</sub>,∞) ),
      ( v<sub>red</sub>, [d<sub>2</sub>,∞) [w<sub>2</sub>,∞) ) }
c<sub>i</sub> = ( v<sub>blue</sub>)
c<sub>2</sub> = ( v<sub>yellow</sub> )
c<sub>3</sub> = ( v<sub>blue</sub>, v<sub>yellow</sub>, v<sub>red</sub>)
</pre></code>
```

This is syntactic sugar, but looks better to a programmer, who fears redundancy. But better still  this notation suggests another simplification, by using ranges of versions. Sets of subsequent, contingent (no holes in between) versions can be represented by ranges with the creating version as *lower* and the invalidating version as *upper* bound . For the intervals of components that persist,i.e. are not invalidated by a subsequent version, we introduce a pseudo version ∞, which is the greatest upper bound and denotes not invalidated by an exisiting version. This is another joy for the programmer, because creating new versions of the aggregate can be done without updating the upper bounds of intervals of unchanged components.

```@raw html
<pre><code>Let

h = { ( v<sub>blue</sub>, [d<sub>0</sub>,d<sub>1</sub>) [w<sub>0</sub>,∞) [d<sub>1</sub>,d<sub>2</sub>) [w<sub>0</sub>,w<sub>1</sub>) [d<sub>2</sub>,∞) [w<sub>0</sub>,w<sub>2</sub>) ),
      ( v<sub>yellow</sub>, [d<sub>1</sub>,d<sub>2</sub>,) [w<sub>1</sub>,∞) ),
      ( v<sub>red</sub>, [d<sub>2</sub>,∞) [w<sub>2</sub>,∞) ) }
c<sub>i</sub> = [v<sub>blue</sub>,v<sub>yellow</sub>)
c<sub>2</sub> = [v<sub>yellow</sub>,v<sub>red</sub>)
c<sub>3</sub> = [v<sub>blue</sub>, v<sub>∞</sub>)
</pre></code>
```

## long transactions 
A transaction is identified by a version and all transaction data depends on it.
* Beginning a transaction on a bitemporal aggregate means creating a version object and 
an interval of validity labelled as not commited.
* New revisions of components are marked as valid from the new version on. Revisions of mutated or deleted components are marked as invalid from the new version.
* Backing out of a transaction means deleting the version object and its depending data: 
    * deleting the new revisions marked as valid from the rollbacked version on
    * retracting the marks of being invalid from the rollbacked version on
* Committing a version means labelling its interval of validity as committed and managing shadowing and overlapping of its interval
of validity whith those of previous versions as is described in the following.

## GiST indexing in POSTGRES for search and guarding bitemporal uniqueness

Database access to bitemporal objects can be supported by GiST (Generalized Search Tree) indices on our

* timestamp-intervals for database and world validity as well as
* revision intervals

So in POSTGRES, for example - hava a look at the UML-Diagram below - , we can access a component (with table name *PART*) by a simple join with filters for

* to locate a history's version by timestamps'inclusion in validity intervals and
* a *PART*'s revision by the version id's inclusion in the *PART_REVISION*'s range of valid versions.

```@raw html
<pre><code>SELECT * FROM histories h
JOIN version v ON v.ref_history = h.id
JOIN validity_interval i ON i.ref_version = v.id 
JOIN <i>PART</i> p ON p.ref_history
JOIN <i>PART_REVISION</i> r ON r.ref_<i>PART</i> = p.id AND r.validity_range  @> v.id
WHERE h.id = :history_id and p.id = :part_id AND i.tsr_world @> TIMESTAMPTZ:ts_world and i.tsr_db @> TIMESTAMPTZ :ts_db
</code></pre>
```

POSTGRES also provides uniqueness constraints for GIST-indices, so that we can guarantee for every history, that its versions validity_intervals are unique, that is non overlapping. Inserting an overlapping interval fails constraint violation exception.

```@raw html
<pre><code>ADD CONSTRAINT bitemp EXCLUDE USING GIST (ref_version WITH =, is_committed WITH =, tsrworld WITH &&, tsrdb WITH &&)`
</code></pre>
```


# A first sketch of a bitemporal UML model for insurance contracts

![UML Model](../assets/BitemporalModel.uxf.png)

Please compare:
https://hdombrovskaya.wordpress.com/2019/07/14/lets-go-bitemporal/
https://www.postgresql.eu/events/pgdayparis2019/sessions/session/2291/slides/171/pgdayparis_2019_msedivy_bitemporality.pdf
